# #' ---
#' title: "JDS - sample data"
#' author: "Jonathan Yu"
#' date: "September 25, 2021"
#' output: 
#'      html_document:
#'           toc: true
#'           toc_depth: 2
#'           toc_float: true
#' ---
#' 

#' Setup

## Libraries
pacman::p_load(tidyverse, ggplot2, survival, tidyverse, here, survminer, survival, CBPS, twang)

## Read data
input.data = here("JDS_data.csv")
df = read_csv(input.data) %>% 
  mutate(
    hcvdr_num = as.factor(hcvdr_num),
    r_Blood = as.factor(r_Blood),
    GENDER = as.factor(GENDER),
    PVD = as.factor(PVD),
    eth_group_cat = as.factor(eth_group_cat),
    don_Blood = as.factor(don_Blood),
    has_diab = as.factor(has_diab)
  )

CBPS.full <- CBPS(hcvdr_num~PRA+r_Blood+AGE+GENDER+KDRI+prev_tx_num+PVD+
                    eth_group_cat+genderd_num+don_Blood+AGE_DON+COLD_ISCH_KI+
                    first_week_dial+LOS+has_diab+SERUM_CREAT+BMI_CALC+TRTREJ1YKI,data=df )
twang.full <- mnps(hcvdr_num~PRA+r_Blood+AGE+GENDER+KDRI+prev_tx_num+PVD+
                     eth_group_cat+genderd_num+don_Blood+AGE_DON+COLD_ISCH_KI+
                     first_week_dial+LOS+has_diab+SERUM_CREAT+BMI_CALC+TRTREJ1YKI,data = df,
                   estimand = "ATE",verbose = FALSE,stop.method = c("es.mean", "ks.mean"),n.trees = 10000)

new.names <- c(GENDER_M = "Gender (M/F)",
               BMI_CALC = "BMI",
               AGE = "Age (Years)",
               eth_group_cat_AfricanAmericans = "Race: AA",
               eth_group_cat_Caucasians = "Race: Caucasians",
               eth_group_cat_Others = "Race: Others",
               has_diab_NoDiabetes = "Diabetes (Y/N)",
               prev_tx_num = "Num of prev TX",
               r_Blood_A = "Blood: A",
               r_Blood_AB = "Blood: AB",
               r_Blood_B = "Blood: B",
               r_Blood_O = "Blood: O",
               PVD_Yes = "PVD",
               first_week_dial = "DGF",
               KDRI = "KDRI",
               LOS = "Length of Stay",
               PRA = "PRA",
               SERUM_CREAT = "Creatinine",
               COLD_ISCH_KI = "Cold Ischemia (hours)",
               TRTREJ1YKI = "Trt rej at 1 yr (Y/N)",
               genderd_num = "Donor Gender (M/F)",
               AGE_DON = "Donor Age (Years)",
               don_Blood_A = "Donor Blood A",
               don_Blood_AB = "Donor Blood AB",
               don_Blood_B = "Donor Blood B",
               don_Blood_O = "Donor Blood O"
)



bal.tab(CBPS.full,disp.means=T,disp.ks=T,disp.v.ratio=T,m.threshold = .25,v.threshold = 2)
bal.tab(twang.full,disp.means=T,disp.ks=T,disp.v.ratio=T,m.threshold = .25,v.threshold = 2,imbalanced.only=T)

twangDiag = bal.table(twang.full, digits = 3)
# write.csv(twangDiag,"twangDiag.csv") # Save twang diagnostics

bal.plot(CBPS.full,which="both",var.name="KDRI")
bal.plot(twang.full,which="both", var.name="KDRI")

love.plot(twang.full,binary="std",threshold=.2, var.names=new.names,
          sample.names = c("Unweighted","Effect size mean","KS mean"))
love.plot(CBPS.full,binary="std",threshold=.2,var.names = new.names)

cbps.invProp <- 1/CBPS.full$fitted.values
cbps.weight <- sapply(1:nrow(full.covariates),function(x){cbps.invProp[x,full.covariates$hcvdr[x]]} )
twang.ESweight <- get.weights(twang.full, stop.method = "es.mean")
full.covariates$cbps.weight <- cbps.weight
full.covariates$twang.ESweight <- twang.ESweight
full.covariates2 <- full.covariates[,c("REGION","CTR_CODE","TRR_ID_CODE","PTIME","PSTATUS","GTIME_KI","GSTATUS_KI",
                                       "ptime_10yr","pstatus_10yr","gtime_10yr","gstatus_10yr",
                                       "hcvdr","hcvdr_num",
                                       "AGE","GENDER","PRA","r_Blood","KDRI",
                                       "prev_tx_num","eth_group_cat","genderd_num",
                                       "don_Blood","AGE_DON","first_week_dial","PVD","COLD_ISCH_KI","CREAT_TRR",
                                       "LOS","has_diab","SERUM_CREAT","BMI_CALC","TRTREJ1Y_KI","twang.ESweight","cbps.weight")]
