libname local 'C:\Users\Jonathan Yu\Dropbox\Aim 1\Data';

PROC IMPORT OUT= work.data_kid 
            DATAFILE= "C:\Users\Jonathan Yu\Dropbox\Aim 1\Data\final_data_aim1.csv" 
            DBMS=CSV REPLACE;
     * RANGE="data$"; 
     GETNAMES=YES;
    * MIXED=NO;
    * SCANTEXT=YES;
    * USEDATE=YES;
    * SCANTIME=YES;
RUN;

/****************************************************************************************/
/*                                 Patient Survival                                     */
/****************************************************************************************/

/* Initial values can be found by fitting */
proc phreg data= work.data_kid ;
class hcvdr_num (ref="0")/param=ref;
model ptime*pstatus(0) = hcvdr_num;
run;

/***--- NON-CLUSTERED EVENTS WITHOUT CURE-RATE PROPORTION ---***/
ods output FitStatistics=work.fit1;
proc nlmixed data=work.data_kid qpoints=170 noad;
	parms eta=-1.4 theta=4.7 gamma=1.4
		beta0  = 7.7 beta1  = 0.98189 beta2  = 0.98189 beta3  = 0.98189
        gamma0 = 0.1 gamma1 = 0.1     gamma2 = 0.1     gamma3 = 0.1 ;

	bounds gamma theta >= 0;

	check = gamma0 + (gamma1 * (hcvdr_num=3)) +  (gamma2 * (hcvdr_num=2)) + (gamma3 * (hcvdr_num=1)) + eta*nu;
	expcheck = exp(-check);
    prob= 1/(1+expcheck);

	XB= beta0 - (beta1*(hcvdr_num=3) + beta2*(hcvdr_num=2) + beta3*(hcvdr_num=1)); 
	lin_pred= XB+nu; 
	alpha= exp(-lin_pred);

	S_t= exp(-(alpha*ptime)**gamma);
	weiblpdf_basehaz= gamma*alpha*((alpha*ptime)**(gamma-1))*S_t;

	if pstatus=0 then loglik = log(prob + (1-prob) * S_t); 
	if pstatus=1 then loglik = log(1-prob) + log(weiblpdf_basehaz); 
	
	model ptime~general(loglik*iptw); 
	random nu~normal(0,theta) subject=ctr_code;
	ods exclude iterhistory parameters;
run;
proc print data=work.fit1 label; 
      format value 8.2; 
run;

/***--- NON-CLUSTERED EVENTS WITH CURE-RATE PROPORTION ---***/
ods output FitStatistics=work.fit2;
proc nlmixed data=work.data_kid qpoints=170 noad;
	parms theta=4.7 gamma=1.4
		beta0  = 7.7 beta1  = 0.98189 beta2  = 0.98189 beta3  = 0.98189;

	bounds gamma theta >= 0;

	XB= beta0 - (beta1*(hcvdr_num=3) + beta2*(hcvdr_num=2) + beta3*(hcvdr_num=1)); 
	lin_pred= XB+nu; 
	alpha= exp(-lin_pred);

	S_t= exp(-(alpha*ptime)**gamma);
	weiblpdf_basehaz= gamma*alpha*((alpha*ptime)**(gamma-1))*S_t;

	if pstatus=0 then loglik = log(S_t); 
	if pstatus=1 then loglik = log(weiblpdf_basehaz); 
	
	model ptime~general(loglik*iptw); 
	random nu~normal(0,theta) subject=ctr_code;
	ods exclude iterhistory parameters;
run;
proc print data=work.fit2 label; 
      format value 8.2; 
run;

/***--- CLUSTERED EVENTS WITHOUT CURE-RATE PROPORTION ---***/
ods output FitStatistics=work.fit3;
proc nlmixed data=work.data_kid qpoints=170 noad;
	parms gamma=1.22
		beta0  = 7.9 beta1  = 0.45 beta2  = 0.49 beta3  = 0.27
        gamma0 = -1.5 gamma1 = -0.15     gamma2 = -4.4     gamma3 = -0.34 ;

	bounds gamma  >= 0;

	check = gamma0 + (gamma1 * (hcvdr_num=3)) +  (gamma2 * (hcvdr_num=2)) + (gamma3 * (hcvdr_num=1));
	expcheck = exp(-check);
    prob= 1/(1+expcheck);

	XB= beta0 - (beta1*(hcvdr_num=3) + beta2*(hcvdr_num=2) + beta3*(hcvdr_num=1)); 
	lin_pred= XB; 
	alpha= exp(-lin_pred);

	S_t= exp(-(alpha*ptime)**gamma);
	weiblpdf_basehaz= gamma*alpha*((alpha*ptime)**(gamma-1))*S_t;

	if pstatus=0 then loglik = log(prob + (1-prob) * S_t); 
	if pstatus=1 then loglik = log(1-prob) + log(weiblpdf_basehaz); 
	
	model ptime~general(loglik*iptw); 
	ods exclude iterhistory parameters;
run;
proc print data=work.fit3 label; 
      format value 8.2; 
run;

/***--- CLUSTERED EVENTS WITH CURE-RATE PROPORTION ---***/
ods output FitStatistics=work.fit4;
proc nlmixed data=work.data_kid qpoints=100;
	parms gamma=1.4
		beta0=7.7  beta1= 0.98189 beta2 = 0.98189 beta3 = 0.98189;

	bounds gamma >= 0;
 
	XB= beta0 - (beta1*(hcvdr_num=3) + beta2*(hcvdr_num=2) + beta3*(hcvdr_num=1)); 
	lin_pred= XB; 
	alpha= 1/exp(lin_pred);

	S_t= exp(-(alpha*ptime)**gamma); 
	weiblpdf_basehaz= gamma*alpha*((alpha*ptime)**(gamma-1))*S_t;

	if pstatus=0 then loglik = log(S_t); 
	if pstatus=1 then loglik = log(weiblpdf_basehaz); 
	
	model ptime~general(loglik*iptw); 
run;
proc print data=work.fit4 label; 
      format value 8.2; 
run;

/****************************************************************************************/
/*                                 Kidney graft Survival                                */
/****************************************************************************************/

proc phreg data= work.data_kid;
class hcvdr_num (ref="0")/param=ref;
model gtime_ki*gstatus_ki(0) = hcvdr_num;
run;
/***--- NON-CLUSTERED EVENTS WITHOUT CURE-RATE PROPORTION ---***/
ods output FitStatistics=local.fit5;
proc nlmixed data=work.data_kid qpoints=170 noad;
	parms beta0 = 7.7 beta1 = 0.35952 beta2 = 0.70572 beta3 = 0.545555
		  gamma0 = 0.1 gamma1 = 0.1 gamma2 = 0.1  gamma3 = 0.1  
		  theta=4.7 
		  g_r01-g_r10 1e-4;

	bounds g_r01 g_r02 g_r03 g_r04 g_r05 g_r06 g_r07 g_r08 g_r09 g_r10 theta  >= 0;

	base_haz = g_r01*g_fail_r1 + g_r02*g_fail_r2 + g_r03*g_fail_r3 + g_r04*g_fail_r4 + g_r05*g_fail_r5
		+ g_r06*g_fail_r6 + g_r07*g_fail_r7 + g_r08*g_fail_r8 + g_r09*g_fail_r9 + g_r10*g_fail_r10; 
	cum_base_haz = g_r01*g_dur_r1 + g_r02*g_dur_r2 + g_r03*g_dur_r3 + g_r04*g_dur_r4 + g_r05*g_dur_r5
		+ g_r06*g_dur_r6 + g_r07*g_dur_r7 + g_r08*g_dur_r8 + g_r09*g_dur_r9 + g_r10*g_dur_r10;
	
	* log-survival prob *;
	mu = beta0*(hcvdr_num=1) + beta1*(hcvdr_num=2)+ beta2*(hcvdr_num=3) + beta3*(hcvdr_num=4) ;
	ll = -cum_base_haz*exp(mu);

	if gstatus_ki=0 then loglik = ll;
	if gstatus_ki=1 then loglik = ll + log(base_haz) + mu;

	model gtime_ki~general(loglik*cbps_weight);

	ods exclude iterhistory parameters;
run;
proc print data=local.fit5 label; 
      format value 8.2; 
run;

/***--- NON-CLUSTERED EVENTS WITH CURE-RATE PROPORTION ---***/
ods output FitStatistics=local.fit6;
proc nlmixed data=work.data_kid qpoints=170 noad;
	parms beta0 = 7.7 beta1 = 0.35952 beta2 = 0.70572 beta3 = 0.545555
		  gamma0 = 0.1 gamma1 = 0.1 gamma2 = 0.1     gamma3 = 0.1  
		  theta=4.7 
		  g_r01-g_r10 1e-4;

	bounds g_r01 g_r02 g_r03 g_r04 g_r05 g_r06 g_r07 g_r08 g_r09 g_r10 theta  >= 0;


	base_haz = g_r01*g_fail_r1 + g_r02*g_fail_r2 + g_r03*g_fail_r3 + g_r04*g_fail_r4 + g_r05*g_fail_r5
		+ g_r06*g_fail_r6 + g_r07*g_fail_r7 + g_r08*g_fail_r8 + g_r09*g_fail_r9 + g_r10*g_fail_r10; 
	cum_base_haz = g_r01*g_dur_r1 + g_r02*g_dur_r2 + g_r03*g_dur_r3 + g_r04*g_dur_r4 + g_r05*g_dur_r5
		+ g_r06*g_dur_r6 + g_r07*g_dur_r7 + g_r08*g_dur_r8 + g_r09*g_dur_r9 + g_r10*g_dur_r10;

	* cure *;
	check = gamma0 + (gamma1 * (hcvdr_num=3)) +  (gamma2 * (hcvdr_num=2)) + (gamma3 * (hcvdr_num=1));
	expcheck = exp(-check);
    prob= 1/(1+expcheck);
	
	* log-survival prob *;
	mu = beta0*(hcvdr_num=1) + beta1*(hcvdr_num=2)+ beta2*(hcvdr_num=3) + beta3*(hcvdr_num=4);
	ll = -cum_base_haz*exp(mu);

	if gstatus_ki=0 then loglik = log(prob + (1-prob) * exp(ll)); 
	if gstatus_ki=1 then loglik = log(1-prob) + log(base_haz) + ll + mu;
	
	model gtime_ki~general(loglik*cbps_weight);

	ods exclude iterhistory parameters;
run;
proc print data=local.fit6 label; 
      format value 8.2; 
run;

/***--- CLUSTERED EVENTS WITHOUT CURE-RATE PROPORTION ---***/
ods output FitStatistics=local.fit7;
proc nlmixed data=work.data_kid qpoints=170 noad;
	parms beta0 = 7.7 beta1 = 0.35952 beta2 = 0.70572 beta3 = 0.545555
		  gamma0 = 0.1 gamma1 = 0.1 gamma2 = 0.1  gamma3 = 0.1  
		  theta=4.7 
		  g_r01-g_r10 1e-4;

	bounds g_r01 g_r02 g_r03 g_r04 g_r05 g_r06 g_r07 g_r08 g_r09 g_r10 theta  >= 0;


	base_haz = g_r01*g_fail_r1 + g_r02*g_fail_r2 + g_r03*g_fail_r3 + g_r04*g_fail_r4 + g_r05*g_fail_r5
		+ g_r06*g_fail_r6 + g_r07*g_fail_r7 + g_r08*g_fail_r8 + g_r09*g_fail_r9 + g_r10*g_fail_r10; 
	cum_base_haz = g_r01*g_dur_r1 + g_r02*g_dur_r2 + g_r03*g_dur_r3 + g_r04*g_dur_r4 + g_r05*g_dur_r5
		+ g_r06*g_dur_r6 + g_r07*g_dur_r7 + g_r08*g_dur_r8 + g_r09*g_dur_r9 + g_r10*g_dur_r10;
	
	* log-survival prob *;
	mu = beta0*(hcvdr_num=1) + beta1*(hcvdr_num=2)+ beta2*(hcvdr_num=3) + beta3*(hcvdr_num=4) + nu;
	ll = -cum_base_haz*exp(mu);

	if gstatus_ki=0 then loglik = ll;
	if gstatus_ki=1 then loglik = ll + log(base_haz) + mu;
	model gtime_ki~general(loglik*cbps_weight);
	random nu~normal(0,theta) subject=ctr_code;
	
	ods exclude iterhistory parameters;
run;
proc print data=local.fit7 label; 
      format value 8.2; 
run;

/***--- CLUSTERED EVENTS WITH CURE-RATE PROPORTION ---***/
ods output FitStatistics=local.fit8;
proc nlmixed data=work.data_kid qpoints=190 noad;
	parms beta0 = 7.7 beta1 = 0.35952 beta2 = 0.70572 beta3 = 0.545555
		  gamma0 = 0.1 gamma1 = 0.1 gamma2 = 0.1     gamma3 = 0.1  
		  theta=0.44 lambda = 0.3 
		  g_r01-g_r10 1e-4;

	bounds g_r01 g_r02 g_r03 g_r04 g_r05 g_r06 g_r07 g_r08 g_r09 g_r10 theta  >= 0;


	base_haz = g_r01*g_fail_r1 + g_r02*g_fail_r2 + g_r03*g_fail_r3 + g_r04*g_fail_r4 + g_r05*g_fail_r5
		+ g_r06*g_fail_r6 + g_r07*g_fail_r7 + g_r08*g_fail_r8 + g_r09*g_fail_r9 + g_r10*g_fail_r10; 
	cum_base_haz = g_r01*g_dur_r1 + g_r02*g_dur_r2 + g_r03*g_dur_r3 + g_r04*g_dur_r4 + g_r05*g_dur_r5
		+ g_r06*g_dur_r6 + g_r07*g_dur_r7 + g_r08*g_dur_r8 + g_r09*g_dur_r9 + g_r10*g_dur_r10;

	* cure *;
	check = gamma0 + (gamma1 * (hcvdr_num=3)) +  (gamma2 * (hcvdr_num=2)) + (gamma3 * (hcvdr_num=1)) + lambda*nu;
	expcheck = exp(-check);
    prob= 1/(1+expcheck);
	
	* log-survival prob *;
	mu = beta0*(hcvdr_num=1) + beta1*(hcvdr_num=2)+ beta2*(hcvdr_num=3) + beta3*(hcvdr_num=4) + nu;
	ll = -cum_base_haz*exp(mu);

	if gstatus_ki=0 then loglik = log(prob + (1-prob) * exp(ll)); 
	if gstatus_ki=1 then loglik = log(1-prob) + log(base_haz) + ll + mu;
	
	model gtime_ki~general(loglik*cbps_weight);
	random nu~normal(0,theta) subject=ctr_code;
	
	ods exclude iterhistory parameters;
run;
proc print data=local.fit8 label; 
      format value 8.2; 
run;
